@@ -3,6 +3,7 @@
 __author__ = "Taylor Oshan Tayoshan@gmail.com"
 
 import copy
+from typing import Optional
 import numpy as np
 import numpy.linalg as la
 from scipy.stats import t
@@ -82,6 +83,9 @@ class GWR(GLM):
                     True to store full n by n hat matrix,
                     False to not store full hat matrix to minimize memory footprint (defalut).
 
+    name_x        : list of strings
+                    Names of independent variables for use in output
+
     Attributes
     ----------
     coords        : array-like
@@ -134,7 +138,7 @@ class GWR(GLM):
     spherical     : boolean
                     True for shperical coordinates (long-lat),
                     False for projected coordinates (defalut).
-                    
+
     hat_matrix    : boolean
                     True to store full n by n hat matrix,
                     False to not store full hat matrix to minimize memory footprint (defalut).
@@ -206,7 +210,7 @@ class GWR(GLM):
 
     def __init__(self, coords, y, X, bw, family=Gaussian(), offset=None,
                  sigma2_v1=True, kernel='bisquare', fixed=False, constant=True,
-                 spherical=False, hat_matrix=False):
+                 spherical=False, hat_matrix=False, name_x=None):
         """
         Initialize class
         """
@@ -229,9 +233,14 @@ class GWR(GLM):
         self.P = None
         self.spherical = spherical
         self.hat_matrix = hat_matrix
+        self.name_x = name_x
 
     def _build_wi(self, i, bw):
 
+        if bw == np.inf:
+            wi = np.ones((self.n))
+            return wi
+
         try:
             wi = Kernel(i, self.coords, bw, fixed=self.fixed,
                         function=self.kernel, points=self.points,
@@ -349,7 +358,7 @@ class GWR(GLM):
                 tr_STS = np.sum(np.array(rslt_list[-2]))
                 CCT = np.array(rslt_list[-1])
                 return GWRResults(self, params, predy, S, CCT, influ, tr_STS,
-                                  w)
+                                  w, self.name_x)
 
 
     def predict(self, points, P, exog_scale=None, exog_resid=None,
@@ -432,6 +441,9 @@ class GWRResults(GLMResults):
                           n*1, final weight used for iteratively re-weighted least
                           sqaures; default is None
 
+    name_x        : list of strings
+                    Names of independent variables for use in output
+
     Attributes
     ----------
     model               : GWR Object
@@ -517,10 +529,10 @@ class GWRResults(GLMResults):
 
     R2                  : float
                           R-squared for the entire model (1- RSS/TSS)
-                          
+
     adj_R2              : float
                           adjusted R-squared for the entire model
-                          
+
     aic                 : float
                           Akaike information criterion
 
@@ -576,11 +588,11 @@ class GWRResults(GLMResults):
     pDev                : float
                           local percent of deviation accounted for; analogous to
                           r-squared for GLM's
-                          
+
     D2                  : float
                           percent deviance explained for GLM, equivaleng to R2 for
                           Gaussian.
-                          
+
     adj_D2              : float
                           adjusted percent deviance explained, equivaleng to adjusted
                           R2 for Gaussian.
@@ -597,10 +609,13 @@ class GWRResults(GLMResults):
                           p*1, predicted values generated by calling the GWR
                           predict method to predict dependent variable at
                           unsampled points ()
+
+    name_x        : list of strings
+                    Names of independent variables for use in output
     """
 
     def __init__(self, model, params, predy, S, CCT, influ, tr_STS=None,
-                 w=None):
+                 w=None, name_x=None):
         GLMResults.__init__(self, model, params, predy, w)
         self.offset = model.offset
         if w is not None:
@@ -611,6 +626,7 @@ class GWRResults(GLMResults):
         self.influ = influ
         self.CCT = self.cov_params(CCT, model.exog_scale)
         self._cache = {}
+        self.name_x=name_x
 
     @cache_readonly
     def W(self):
@@ -1054,7 +1070,7 @@ class GWRResults(GLMResults):
     @cache_readonly
     def use_t(self):
         return None
-    
+
     def get_bws_intervals(self, selector, level=0.95):
         """
         Computes bandwidths confidence interval (CI) for GWR.
@@ -1064,12 +1080,12 @@ class GWRResults(GLMResults):
         Returns a tuple with lower and upper bound of the bw CI.
         e.g. (100, 300)
         """
-        
+
         try:
             import pandas as pd
         except ImportError:
             return
-        
+
         #Get AICcs and associated bw from the last iteration of back-fitting and make a DataFrame
         aiccs = pd.DataFrame(list(zip(*selector.sel_hist))[1],columns=["aicc"])
         aiccs['bw'] = list(zip(*selector.sel_hist))[0]
@@ -1087,7 +1103,7 @@ class GWRResults(GLMResults):
         #Get bw boundaries
         interval = (aiccs.iloc[:index,:].bw.min(),aiccs.iloc[:index,:].bw.max())
         return interval
-    
+
 
     def local_collinearity(self):
         """
@@ -1214,7 +1230,13 @@ class GWRResults(GLMResults):
         init_sd = np.std(self.params, axis=0)
         SDs = []
 
-        for x in range(n_iters):
+        try:
+            from tqdm.auto import tqdm  # if they have it, let users have a progress bar
+        except ImportError:
+            def tqdm(x, desc=''):  # otherwise, just passthrough the range
+                return x
+
+        for x in tqdm(range(n_iters), desc='Testing'):
             temp_coords = np.random.permutation(self.model.coords)
             temp_sel.coords = temp_coords
             temp_bw = temp_sel.search(**search_params)
@@ -1237,20 +1259,36 @@ class GWRResults(GLMResults):
             predictions = np.sum(P * self.params, axis=1).reshape((-1, 1))
         return predictions
 
-    def summary(self):
+    def summary(self, as_str: bool = False) -> Optional[str]:
         """
         Print out GWR summary
+
+        Parameters
+        ----------
+        as_str        : bool
+                        optional parameters to specify that summary results
+                        should be returned as str and not printed to stdout
+
+        Returns
+        -------
+
+        summary        : Optional[str]
+                        optional GWR summary string if `as_str` is True
         """
         summary = summaryModel(self) + summaryGLM(self) + summaryGWR(self)
+
+        if as_str:
+            return summary
+
         print(summary)
-        return
+        return None
 
 
 class GWRResultsLite(object):
     """
     Lightweight GWR that computes the minimum diagnostics needed for bandwidth
     selection.
-    
+
     See FastGWR,Li et al., 2019, IJGIS.
 
     Parameters
@@ -1372,6 +1410,9 @@ class MGWR(GWR):
                     hat matrices R (n,n,k) and model hat matrix S (n,n).
                     False (default) for computing MGWR inference on the fly.
 
+    name_x        : list of strings
+                    Names of independent variables for use in output
+
     Attributes
     ----------
     coords        : array-like
@@ -1448,6 +1489,9 @@ class MGWR(GWR):
                     observations from each calibration point: one for each
                     covariate (k)
 
+    name_x        : list of strings
+                    Names of independent variables for use in output
+
     Examples
     --------
 
@@ -1477,7 +1521,7 @@ class MGWR(GWR):
 
     def __init__(self, coords, y, X, selector, sigma2_v1=True,
                  kernel='bisquare', fixed=False, constant=True,
-                 spherical=False, hat_matrix=False):
+                 spherical=False, hat_matrix=False, name_x=None):
         """
         Initialize class
         """
@@ -1499,6 +1543,7 @@ class MGWR(GWR):
         self.exog_resid = None
         self.exog_scale = None
         self_fit_params = None
+        self.name_x = name_x
 
     def _chunk_compute_R(self, chunk_id=0):
         """
@@ -1558,15 +1603,15 @@ class MGWR(GWR):
         """
         Compute MGWR inference by chunk to reduce memory footprint.
         See Li and Fotheringham, 2020, IJGIS.
-        
+
         Parameters
         ----------
 
         n_chunks      : integer, optional
-                        A number of chunks parameter to reduce memory usage. 
+                        A number of chunks parameter to reduce memory usage.
                         e.g. n_chunks=2 should reduce overall memory usage by 2.
         pool          : A multiprocessing Pool object to enable parallel fitting; default is None.
-                        
+
         Returns
         -------
                       : MGWRResults
@@ -1601,22 +1646,22 @@ class MGWR(GWR):
             R = np.hstack(rslt_list[2])
         else:
             R = None
-        return MGWRResults(self, params, predy, CCT, ENP_j, w, R)
-    
-    
+        return MGWRResults(self, params, predy, CCT, ENP_j, w, R, self.name_x)
+
+
     def exact_fit(self):
         """
         A closed-form solution to MGWR estimates and inference,
         the backfitting in self.fit() will converge to this solution.
-        
+
         Note: this would require large memory when n > 5,000.
         See Li and Fotheringham, 2020, IJGIS, pg.4.
-        
+
         Returns
         -------
                       : MGWRResults
         """
-        
+
         P = []
         Q = []
         I = np.eye(self.n)
@@ -1630,7 +1675,7 @@ class MGWR(GWR):
                     Pj.append(Aj)
             P.append(Pj)
             Q.append([Aj])
-    
+
         P = np.block(P)
         Q = np.block(Q)
         R = np.linalg.solve(P, Q)
@@ -1647,10 +1692,10 @@ class MGWR(GWR):
         CCT = np.zeros((self.n,self.k))
         for j in range(self.k):
             CCT[:, j] = ((R[:, :, j] / self.X[:, j].reshape(-1, 1))**2).sum(axis=1)
-        
+
         return MGWRResults(self, params, predy, CCT, ENP_j, w, R)
-    
-    
+
+
     def predict(self):
         '''
         Not implemented.
@@ -1686,6 +1731,9 @@ class MGWRResults(GWRResults):
                           n*1, final weight used for iteratively re-weighted least
                           sqaures; default is None
 
+    name_x              : list of strings
+                           Names of independent variables for use in output
+
     Attributes
     ----------
     model               : GWR Object
@@ -1778,7 +1826,7 @@ class MGWRResults(GWRResults):
 
     R2                  : float
                           R-squared for the entire model (1- RSS/TSS)
-                          
+
     adj_R2              : float
                           adjusted R-squared for the entire model
 
@@ -1823,7 +1871,7 @@ class MGWRResults(GWRResults):
 
     """
 
-    def __init__(self, model, params, predy, CCT, ENP_j, w, R):
+    def __init__(self, model, params, predy, CCT, ENP_j, w, R, x_name=None):
         """
         Initialize class
         """
@@ -1833,6 +1881,7 @@ class MGWRResults(GWRResults):
         if model.hat_matrix:
             self.S = np.sum(self.R, axis=2)
         self.predy = predy
+        self.x_name = x_name
 
     @cache_readonly
     def tr_S(self):
@@ -1956,7 +2005,7 @@ class MGWRResults(GWRResults):
     @cache_readonly
     def predictions(self):
         raise NotImplementedError('Not yet implemented for MGWR')
-    
+
     #Function for getting BWs intervals
     def get_bws_intervals(self, selector, level=0.95):
         """
@@ -1965,14 +2014,14 @@ class MGWRResults(GWRResults):
         Details are in Li et al. (2020) Annals of AAG
 
         Returns a list of confidence intervals. e.g. [(40, 60), (100, 180), (150, 300)]
-            
+
         """
         intervals = []
         try:
             import pandas as pd
         except ImportError:
             return
-        
+
         for j in range(self.k):
             #Get AICcs and associated bw from the last iteration of back-fitting and make a DataFrame
             aiccs = pd.DataFrame(list(zip(*selector.sel_hist[-self.k+j]))[1],columns=["aicc"])
@@ -2088,7 +2137,14 @@ class MGWRResults(GWRResults):
         init_sd = np.std(self.params, axis=0)
         SDs = []
 
-        for x in range(n_iters):
+        try:
+            from tqdm.auto import tqdm  # if they have it, let users have a progress bar
+        except ImportError:
+
+            def tqdm(x, desc=''):  # otherwise, just passthrough the range
+                return x
+
+        for x in tqdm(range(n_iters), desc='Testing'):
             temp_coords = np.random.permutation(self.model.coords)
             temp_sel.coords = temp_coords
             temp_sel.search(**search_params)
@@ -2099,10 +2155,26 @@ class MGWRResults(GWRResults):
         p_vals = (np.sum(np.array(SDs) > init_sd, axis=0) / float(n_iters))
         return p_vals
 
-    def summary(self):
+    def summary(self, as_str: bool=False) -> Optional[str]:
         """
         Print out MGWR summary
+
+        Parameters
+        ----------
+        as_str        : bool
+                        optional parameters to specify that summary results
+                        should be returned as str and not printed to stdout
+
+        Returns
+        -------
+
+        summary        : Optional[str]
+                        optional MGWR summary string if `as_str` is True
         """
         summary = summaryModel(self) + summaryGLM(self) + summaryMGWR(self)
+
+        if as_str:
+            return summary
+
         print(summary)
         return
